require 'Qt4'
require 'hornetseye'
require 'qrc_visiongui'
require 'ui_calibrateintropage'

include Hornetseye

class CalibrateDisplay < XvWidget

  def initialize( cx, cy, parent = nil, grab_port_now = true )
    super parent, grab_port_now
    @cx, @cy = cx, cy
  end

  def paintEvent( e )
    super e
    p = Qt::Painter.new self
    p.setRenderHint Qt::Painter::Antialiasing, false
    p.pen = Qt::Pen.new Qt::Color.new( 255, 0, 0 )
    p.drawEllipse @cx - 5, @cy - 5, 10, 10
    p.drawLine @cx - 5, @cy, @cx + 5, @cy
    p.drawLine @cx, @cy - 5, @cx, @cy + 5
    p.end
  end

end

class CalibrateIntroPage < Qt::WizardPage

  slots 'step_size(double)'
  slots 'x(double)'
  slots 'y(double)'
  slots 'z(double)'
  signals 'step_size_changed(double)'
  signals 'x_changed(double)'
  signals 'y_changed(double)'
  signals 'z_changed(double)'

  attr_reader :xvideo

  def initialize( title, w, h, cx, cy, parent = nil )
    super parent
    setTitle title
    @ui = Ui::CalibrateIntroPage.new
    @ui.setupUi self
    @xvideo = CalibrateDisplay.new cx, cy, nil, false
    @xvideo.resize w, h
    @ui.scroll_area.widget = @xvideo
    connect @ui.step_spin, SIGNAL('valueChanged(double)'),
      self, SLOT('step_size(double)')
    connect @ui.x_spin, SIGNAL('valueChanged(double)'),
      self, SIGNAL('x_changed(double)')
    connect @ui.y_spin, SIGNAL('valueChanged(double)'),
      self, SIGNAL('y_changed(double)')
    connect @ui.z_spin, SIGNAL('valueChanged(double)'),
      self, SIGNAL('z_changed(double)')
  end

  def step_size( value )
    emit step_size_changed( value )
    @ui.step_spin.value = value if @ui.step_spin.value != value
    @ui.x_spin.singleStep = value
    @ui.y_spin.singleStep = value
    @ui.z_spin.singleStep = value
  end

  def x( value )
    @ui.x_spin.value = value if @ui.x_spin.value != value
  end

  def y( value )
    @ui.y_spin.value = value if @ui.y_spin.value != value
  end

  def z( value )
    @ui.z_spin.value = value if @ui.z_spin.value != value
  end

end

class CalibrateWizard < Qt::Wizard

  slots 'setup_page(int)'
  slots 'step_size(double)'
  slots 'x(double)'
  slots 'y(double)'
  slots 'z(double)'

  attr_reader :xh, :yh, :zh, :th

  def initialize( video, controller, dt, t_offset, t_delta, parent = nil )
    super parent, Qt::DialogType
    setWindowModality Qt::WindowModal
    points = [ [ 0.25, 0.25 ],
               [ 0.75, 0.25 ],
               [ 0.75, 0.75 ],
               [ 0.25, 0.25 ] ]
    points.each_with_index do |point,i|
      cx = ( point[ 0 ] * video.width  ).to_i
      cy = ( point[ 1 ] * video.height ).to_i
      page = CalibrateIntroPage.new "Calibration Step #{i+1}/#{points.size}",
        video.width, video.height, cx, cy
      connect page, SIGNAL('step_size_changed(double)'),
        self, SLOT('step_size(double)')
      connect page, SIGNAL('x_changed(double)'),
        self, SLOT('x(double)')
      connect page, SIGNAL('y_changed(double)'),
        self, SLOT('y(double)')
      connect page, SIGNAL('z_changed(double)'),
        self, SLOT('z(double)')
      addPage page
    end
    connect self, SIGNAL('currentIdChanged(int)'),
      self, SLOT('setup_page(int)')
    @video, @controller, @dt = video, controller, dt
    @t_offset, @t_delta = t_offset, t_delta
    @xvideo, @timer = nil, 0
    @step_size, @x, @y, @z, @t = 0.1, 0.0, 0.0, 20.0, Time.new.to_f
    @xh = [ @x ] * points.size
    @yh = [ @y ] * points.size
    @zh = [ @z ] * points.size
    @th = [ @t ] * points.size
  end

  def timerEvent( e )
    @xvideo.write @video.read if e.timerId == @timer
  end

  def accept
    super
    setup_page -1
  end

  def setup_page( index )
    if @xvideo != nil
      @xvideo.releasePort 
      @xvideo = nil
    end
    if index >= 0
      @th[ index ] = Time.new.to_f
      p = page index
      p.step_size @step_size
      p.x @x
      p.y @y
      p.z @z
      @xvideo = p.xvideo
      @xvideo.grabPort
      @timer = startTimer 0 if @timer == 0
    else
      if @timer != 0
        killTimer @timer
        @timer = 0
      end
    end
  end

  def step_size( value )
    @step_size = value
  end

  def x( value )
    @xh[ currentId ] = value
    @th[ currentId ] = Time.new.to_f
    @x = value
    @controller.drive @x, @y, @z, ( ( @dt - @t_offset ) / @t_delta ).to_i
  end

  def y( value )
    @yh[ currentId ] = value
    @th[ currentId ] = Time.new.to_f
    @y = value
    @controller.drive @x, @y, @z, ( ( @dt - @t_offset ) / @t_delta ).to_i
  end

  def z( value )
    @zh[ currentId ] = value
    @th[ currentId ] = Time.new.to_f
    @z = value
    @controller.drive @x, @y, @z, ( ( @dt - @t_offset ) / @t_delta ).to_i
  end

end
