// g++ -shared -fPIC -shared -I/usr/lib/ruby/1.8/x86_64-linux -o LabelingExtension.so LabelingExtension.cc

// Fast Connected Commponents  (Ruby Extension)
// Version: 2.0
// Date: 18/02/2009
// Lastest modification: 05/06/2009
// Hussein S Abdul-Rahamn

#include <ruby.h>
#include <stdlib.h>

#define Labels_Array_Size 100000


VALUE mHornetseye;
VALUE cMalloc;
VALUE cMultiArray, dMultiArray;

void deleteMallocObject( void *ptr )
{
  xfree( ptr );
}

//************************************* Function ***********************************************
//**********************************************************************************************
int* component_areas_local(unsigned char* DataIn, int w,int h)
{
  int no_pixels;
  no_pixels = w*h;
  int *areas;

  areas = (int *)malloc(Labels_Array_Size*sizeof(int));
  
  for(int y=0;y<Labels_Array_Size;y++)
    areas[y]=0;


  for (int i=0;i<no_pixels;i++)
  {
    areas[DataIn[i]]++;
  }

  return areas;
}

//************************************* Function ***********************************************
//**********************************************************************************************
VALUE component_areas(VALUE self, VALUE rbData, VALUE offset, VALUE w,VALUE h)
{
  int *areas;
  unsigned char* DataIn;
  int width,height,no_pixels;
  

  width = FIX2INT(w);
  height= FIX2INT(h);
  no_pixels = width*height;

  Data_Get_Struct( rbData, unsigned char, DataIn );
  DataIn += NUM2INT( offset );
  areas = ALLOC_N(int,Labels_Array_Size);
  
  for(int y=0;y<Labels_Array_Size;y++)
    areas[y]=0;


  for (int i=0;i<no_pixels;i++)
  {
    areas[DataIn[i]]++;
  }

  return Data_Wrap_Struct( cMalloc, 0, deleteMallocObject, (void *)areas );
}

//************************************* Function ***********************************************
//**********************************************************************************************
VALUE component_centre_of_gravity(VALUE self,VALUE rbData,VALUE offset,VALUE w,VALUE h)
{
  int *i_index_sum,*j_index_sum,*areas;
  int *centre_index,*i_centre_index,*j_centre_index;
  unsigned char* DataIn;
  int width,height,no_pixels;
  

  width = FIX2INT(w);
  height= FIX2INT(h);
  no_pixels = width*height;

  i_index_sum = (int *)malloc(Labels_Array_Size*sizeof(int));
  j_index_sum = (int *)malloc(Labels_Array_Size*sizeof(int));
  i_centre_index = (int *)malloc(Labels_Array_Size*sizeof(int));
  j_centre_index = (int *)malloc(Labels_Array_Size*sizeof(int));
  areas = (int *)malloc(Labels_Array_Size*sizeof(int));

  Data_Get_Struct( rbData, unsigned char, DataIn );
  DataIn += NUM2INT(offset);
  centre_index = ALLOC_N(int,Labels_Array_Size);

  //areas = component_areas_local(DataIn,width,height);
  
  for(int y=0;y<Labels_Array_Size;y++)
  {
    i_index_sum[y]=0;
    j_index_sum[y]=0;
    centre_index[y]=0;
    i_centre_index[y]=0;
    j_centre_index[y]=0;
    areas[y]= 0;
  }


  for (int i=0;i<no_pixels;i++)
  {
    i_index_sum[DataIn[i]] += i/width;
    j_index_sum[DataIn[i]] += i%width;
    areas[DataIn[i]]++;
  }

  for (int j=0;j<Labels_Array_Size;j++)
  {
    if(areas[j] !=0 )
    {
       i_centre_index[j] = (i_index_sum[j] / areas[j]);
       j_centre_index[j] = (j_index_sum[j] / areas[j]);
    }
    else
    {
      i_centre_index[j] = 0;
      j_centre_index[j] = 0;
    }
  }

  for (int j=0;j<Labels_Array_Size;j++)
  {
    centre_index[j] = i_centre_index[j]*width + j_centre_index[j];
  }

  free(i_index_sum);
  free(j_index_sum);
  free(i_centre_index);
  free(j_centre_index);

  return Data_Wrap_Struct( cMalloc, 0, deleteMallocObject, (void *)centre_index );
}


//*********************************New Function ******************************
//****************************************************************************
VALUE connected_component(VALUE self, VALUE rbData, VALUE offset, VALUE w,VALUE h)
{
  unsigned char* DataIn;
  int true_labels[Labels_Array_Size],width,height,tmp_label,tmp_label1;;
  unsigned char l_n[4],l_index[4];
  unsigned char* DataOut;
  int available_label = 0;
  int l_count=0;
  int row, c_l1, c_l2, c_l3, tmp, s_index,tt=0;
  bool s_index_flag,loop_flag;
  int no_pixels;
  
  width = FIX2INT(w);
  height= FIX2INT(h);
  no_pixels = width*height;
  
  Data_Get_Struct( rbData, unsigned char, DataIn );
  DataIn += NUM2INT(offset);
  DataOut = ALLOC_N(unsigned char,no_pixels);

  for(int y=0;y<no_pixels;y++)
    DataOut[y]=0;

  row = width;
  
  for(int o=0;o<Labels_Array_Size;o++)
  {
    true_labels[o]=0;
  }

  for(int i=row;i<no_pixels;i++)
 {
    
    if(DataIn[i] == 1)
    {
//       if(i<0)
//       {
//         l_n[0] = 0;
//         l_n[1] = 0;
//         l_n[2] = 0;
//         if(i ==0)
//           l_n[3] = 0;
//         else
//           l_n[3] = DataOut[i-1];
//       }
      if( (i%row)==0)
      {
        
        l_n[0] = 0;
        l_n[1] = DataOut[i-row];
        l_n[2] = DataOut[i+1-row];
        l_n[3] = 0;
      }
      else if( (i%(row))==row-1)
      {
        DataOut[0] = 100;
        l_n[0] = DataOut[i-1-row];
        l_n[1] = DataOut[i-row];
        l_n[2] = 0;
        l_n[3] = DataOut[i-1];
      }
      else
      {
        l_n[0] = DataOut[i-1-row];
        l_n[1] = DataOut[i-row];
        l_n[2] = DataOut[i+1-row];
        l_n[3] = DataOut[i-1];

      }

      l_count = 0;
      if( (l_n[0]==0) && (l_n[1]==0) && (l_n[2]==0) && (l_n[3]==0))
      {
        available_label += 1;
        DataOut[i] = available_label;
        true_labels[available_label] = available_label;
      }
      else
      {
        c_l1 = 0;
        c_l2 = 0;
        c_l3 = 0;
        tmp = 0;
        l_index[0] =100; l_index[1]=100; l_index[2]=100; l_index[3]=100;
        s_index = 0;
        s_index_flag = true;
        for(int t=0;t<4;t++)
        {
          if((l_n[t] != 0) && (l_n[t] != c_l1) && (l_n[t] != c_l2) && (l_n[t] != c_l3))
          {
            c_l3 = c_l2;
            c_l2 = c_l1;
            c_l1 = l_n[t];
            l_count +=1;
            l_index[tmp] = t;
            tmp +=1;
            if(!s_index_flag )
              s_index = (l_n[t] < l_n[s_index])? t : s_index;
            else
            {
              s_index_flag = false;
              s_index = t;
            }

          }
        }
        if(l_count == 1)
          DataOut[i] = true_labels[l_n[s_index]];
        else
        {
          DataOut[i] = true_labels[l_n[s_index]];
          tt = 0;
          // Hussein 20/02/2009
          tmp_label = l_n[s_index];
          while(true_labels[tmp_label] != tmp_label)
          {
            tmp_label = true_labels[tmp_label];
          }
          while (l_index[tt] != 100)
          {
            tmp_label1 = l_n[l_index[tt]];
            while(true_labels[tmp_label1] != tmp_label1)
            {
              tmp_label1 = true_labels[tmp_label1];
            }

            if( tmp_label > tmp_label1)
              true_labels[tmp_label]=true_labels[tmp_label1];
            else
              true_labels[tmp_label1] = true_labels[tmp_label];

            tt +=1;
          }
        }
 
      }
    }
  }

  for(int ff=1;ff<=available_label;ff++)
  {
    loop_flag = true;
    while (loop_flag)
    {
      if( true_labels[ff] == true_labels[true_labels[ff]])
        loop_flag = false;
      else
        true_labels[ff] = true_labels[true_labels[ff]];
    }
  }
  for(int i=0;i<no_pixels;i++)
  {
    if(DataOut[i] !=0)
      DataOut[i] = true_labels[DataOut[i]];
  }
  // return DataOut;
  return Data_Wrap_Struct( cMalloc, 0, deleteMallocObject, (void *)DataOut );
}


//*************************** New Function **************************************
// *********************************************************************************
VALUE connected_component_1(VALUE self, VALUE rbData, VALUE offset, VALUE w,VALUE h)
{
  unsigned char* DataIn;
  int true_labels[Labels_Array_Size],objects[Labels_Array_Size],width,height;
  unsigned char l_n[4],l_index[4],tmp_label,tmp_label1;
  unsigned char* DataOut;
  int available_label = 0;
  int l_count=0;
  int row, c_l1, c_l2, c_l3, tmp, s_index,tt=0;
  bool s_index_flag,loop_flag;
  int no_pixels;
  
  width = FIX2INT(w);
  height= FIX2INT(h);
  no_pixels = width*height;
  
  Data_Get_Struct( rbData, unsigned char, DataIn );
  DataIn += NUM2INT(offset);
  DataOut = ALLOC_N(unsigned char,no_pixels);

  for(int y=0;y<no_pixels;y++)
    DataOut[y]=0;

  row = width;
  
  for(int o=0;o<Labels_Array_Size;o++)
  {
    true_labels[o]=0;
    objects[o] =0;
  }

  for(int i=row;i<no_pixels;i++)
 {
    
    if(DataIn[i] == 1)
    {
//       if(i<0)
//       {
//         l_n[0] = 0;
//         l_n[1] = 0;
//         l_n[2] = 0;
//         if(i ==0)
//           l_n[3] = 0;
//         else
//           l_n[3] = DataOut[i-1];
//       }
      if( (i%row)==0)
      {
        
        l_n[0] = 0;
        l_n[1] = DataOut[i-row];
        l_n[2] = DataOut[i+1-row];
        l_n[3] = 0;
      }
      else if( (i%(row))==row-1)
      {
        DataOut[0] = 100;
        l_n[0] = DataOut[i-1-row];
        l_n[1] = DataOut[i-row];
        l_n[2] = 0;
        l_n[3] = DataOut[i-1];
      }
      else
      {
        l_n[0] = DataOut[i-1-row];
        l_n[1] = DataOut[i-row];
        l_n[2] = DataOut[i+1-row];
        l_n[3] = DataOut[i-1];

      }

      l_count = 0;
      if( (l_n[0]==0) && (l_n[1]==0) && (l_n[2]==0) && (l_n[3]==0))
      {
        available_label += 1;
        DataOut[i] = available_label;
        true_labels[available_label] = available_label;
      }
      else
      {
        c_l1 = 0;
        c_l2 = 0;
        c_l3 = 0;
        tmp = 0;
        l_index[0] =100; l_index[1]=100; l_index[2]=100; l_index[3]=100;
        s_index = 0;
        s_index_flag = true;
        for(int t=0;t<4;t++)
        {
          if((l_n[t] != 0) && (l_n[t] != c_l1) && (l_n[t] != c_l2) && (l_n[t] != c_l3))
          {
            c_l3 = c_l2;
            c_l2 = c_l1;
            c_l1 = l_n[t];
            l_count +=1;
            l_index[tmp] = t;
            tmp +=1;
            if(!s_index_flag )
              s_index = (l_n[t] < l_n[s_index])? t : s_index;
            else
            {
              s_index_flag = false;
              s_index = t;
            }

          }
        }
        if(l_count == 1)
          DataOut[i] = true_labels[l_n[s_index]];
        else
        {
          DataOut[i] = true_labels[l_n[s_index]];
          tt = 0;
          tmp_label = l_n[s_index];
          while(true_labels[tmp_label] != tmp_label)
          {
            tmp_label = true_labels[tmp_label];
          }
          while (l_index[tt] != 100)
          {
            //if(l_index[tt] != s_index)  //19/02/2009
           // new modifications 19/02/2009 13:46

            //Hussein 20/02/2009 "commenting the if following if statment"
//             if(true_labels[l_n[l_index[tt]]] == l_n[l_index[tt]])
//               true_labels[l_n[l_index[tt]]] = true_labels[tmp_label];
//             else
//             {
            tmp_label1 = l_n[l_index[tt]];
            while(true_labels[tmp_label1] != tmp_label1)
            {
              tmp_label1 = true_labels[tmp_label1];
            }

            if( tmp_label > tmp_label1)
              true_labels[tmp_label]=true_labels[tmp_label1];
            else
              true_labels[tmp_label1] = true_labels[tmp_label];
//             }

            tt +=1;
          }
        }
 
      }
    }
  }

  for(int ff=available_label;ff>=1;ff--)
  {
    loop_flag = true;
    while (loop_flag)
    {
      if( true_labels[ff] == true_labels[true_labels[ff]])
        loop_flag = false;
      else
        true_labels[ff] = true_labels[true_labels[ff]];
    }
  }
  tmp = 1;
  for (int ff=1;ff<=available_label;ff++)
  {
    if( true_labels[ff] == ff)
    {
      objects[ff] = tmp;
      tmp++;
    }
  }
  for(int i=0;i<no_pixels;i++)
  {
    if(DataOut[i] !=0)
      // Hussein 26/02/09
      DataOut[i] = objects[true_labels[DataOut[i]]];
  }
     // Hussein 26/02/09 commenting the following for statement
//   for(int i=0;i<no_pixels;i++)
//   {
//     if(DataOut[i] !=0)
//       DataOut[i] = objects[DataOut[i]];
//   }

  // return DataOut;
  return Data_Wrap_Struct( cMalloc, 0, deleteMallocObject, (void *)DataOut );
}



//*************************** New Function ************************************** *********************************************************************************
VALUE connected_component_2(VALUE self, VALUE rbData, VALUE offset, VALUE w,VALUE h,VALUE wi,VALUE wf,VALUE hi,VALUE hf)
{

  int true_labels[Labels_Array_Size],objects[Labels_Array_Size],width,height,start_index,end_index;
  int no_pixels,i,j,xi,xf,yi,yf;
  int available_label = 0;
  int l_count=0;
  int row, c_l1, c_l2, c_l3, tmp, s_index,tt=0;

  unsigned char l_n[4],l_index[4],tmp_label,tmp_label1;
  unsigned char* DataIn;
  unsigned char* DataOut;

  bool s_index_flag,loop_flag;

  
  width = FIX2INT(w);
  height= FIX2INT(h);
  xi = FIX2INT(wi);
  xf = FIX2INT(wf);
  yi = FIX2INT(hi);
  yf = FIX2INT(hf);
  no_pixels = width*height;
  //  05/06/2009: Commenting the following 2 lines:
  //start_index = yi*height + xi;
  //end_index = yf*height + xf;

  // 05/06/2009
  start_index = yi*width + xi;
  end_index = yf*width + xf;
  
  Data_Get_Struct( rbData, unsigned char, DataIn );
  DataIn += NUM2INT( offset );
  DataOut = ALLOC_N(unsigned char,no_pixels);

  for(int y=0;y<no_pixels;y++)
    DataOut[y]=0;

  row = width;
  
  for(int o=0;o<Labels_Array_Size;o++)
    {
      true_labels[o]=0;
      objects[o] =0;
    }

  j = start_index;

  for( int u=yi;u<=yf;u++)
    {
      i = j;
      for (int v=xi;v<=xf;v++)
        {
          if(DataIn[i] == 1)
            {
              if( (v==xi))
                {
        
                  l_n[0] = 0;
                  l_n[1] = DataOut[i-row];
                  l_n[2] = DataOut[i+1-row];
                  l_n[3] = 0;
                }
              else if( v==xf)
                {
                  DataOut[0] = 100;
                  l_n[0] = DataOut[i-1-row];
                  l_n[1] = DataOut[i-row];
                  l_n[2] = 0;
                  l_n[3] = DataOut[i-1];
                }
              else
                {
                  l_n[0] = DataOut[i-1-row];
                  l_n[1] = DataOut[i-row];
                  l_n[2] = DataOut[i+1-row];
                  l_n[3] = DataOut[i-1];
                }

              l_count = 0;
              if( (l_n[0]==0) && (l_n[1]==0) && (l_n[2]==0) && (l_n[3]==0))
                {
                  available_label += 1;
                  DataOut[i] = available_label;
                  true_labels[available_label] = available_label;
                }
              else
                {
                  c_l1 = 0;
                  c_l2 = 0;
                  c_l3 = 0;
                  tmp = 0;
                  l_index[0] =100; l_index[1]=100; l_index[2]=100; l_index[3]=100;
                  s_index = 0;
                  s_index_flag = true;
                  for(int t=0;t<4;t++)
                    {
                      if((l_n[t] != 0) && (l_n[t] != c_l1) && (l_n[t] != c_l2) && (l_n[t] != c_l3))
                        {
                          c_l3 = c_l2;
                          c_l2 = c_l1;
                          c_l1 = l_n[t];
                          l_count +=1;
                          l_index[tmp] = t;
                          tmp +=1;
                          if(!s_index_flag )
                            s_index = (l_n[t] < l_n[s_index])? t : s_index;
                          else
                            {
                              s_index_flag = false;
                              s_index = t;
                            }
                        }
                    }
                  if(l_count == 1)
                    DataOut[i] = true_labels[l_n[s_index]];
                  else
                    {
                      DataOut[i] = true_labels[l_n[s_index]];
                      tt = 0;
                      tmp_label = l_n[s_index];
                      while(true_labels[tmp_label] != tmp_label)
                        {
                          tmp_label = true_labels[tmp_label];
                        }
                      while (l_index[tt] != 100)
                        {
                          tmp_label1 = l_n[l_index[tt]];
                          while(true_labels[tmp_label1] != tmp_label1)
                            {
                              tmp_label1 = true_labels[tmp_label1];
                            }

                          if( tmp_label > tmp_label1)
                            true_labels[tmp_label]=true_labels[tmp_label1];
                          else
                            true_labels[tmp_label1] = true_labels[tmp_label];

                          tt +=1;
                        } 
                    }
 
                }
            }
          i++;
        }
        j += width;
    }

/*  for(int i=start_index;i<=end_index;i++)
 {
    ii = i%row;
    jj = i/row;
    if( (ii>=xi) && (ii<=xf) &&(jj>=yi) &&(jj<=yf))
    {
      if(DataIn[i] == 1)
      {
//         if(i<0)
//         {
//           l_n[0] = 0;
//           l_n[1] = 0;
//           l_n[2] = 0;
//           if(i ==0)
//           l_n[3] = 0;
//           else
//           l_n[3] = DataOut[i-1];
//         }
        if( (ii==xi))
        {
        
          l_n[0] = 0;
          l_n[1] = DataOut[i-row];
          l_n[2] = DataOut[i+1-row];
          l_n[3] = 0;
        }
        else if( (ii)==xf)
        {
          DataOut[0] = 100;
          l_n[0] = DataOut[i-1-row];
          l_n[1] = DataOut[i-row];
          l_n[2] = 0;
          l_n[3] = DataOut[i-1];
        }
        else
        {
          l_n[0] = DataOut[i-1-row];
          l_n[1] = DataOut[i-row];
          l_n[2] = DataOut[i+1-row];
          l_n[3] = DataOut[i-1];

        }

        l_count = 0;
        if( (l_n[0]==0) && (l_n[1]==0) && (l_n[2]==0) && (l_n[3]==0))
        {
          available_label += 1;
          DataOut[i] = available_label;
          true_labels[available_label] = available_label;
        }
        else
        {
          c_l1 = 0;
          c_l2 = 0;
          c_l3 = 0;
          tmp = 0;
          l_index[0] =100; l_index[1]=100; l_index[2]=100; l_index[3]=100;
          s_index = 0;
          s_index_flag = true;
          for(int t=0;t<4;t++)
          {
            if((l_n[t] != 0) && (l_n[t] != c_l1) && (l_n[t] != c_l2) && (l_n[t] != c_l3))
            {
              c_l3 = c_l2;
              c_l2 = c_l1;
              c_l1 = l_n[t];
              l_count +=1;
              l_index[tmp] = t;
              tmp +=1;
              if(!s_index_flag )
                s_index = (l_n[t] < l_n[s_index])? t : s_index;
              else
              {
                s_index_flag = false;
                s_index = t;
              }

            }
          }
          if(l_count == 1)
            DataOut[i] = true_labels[l_n[s_index]];
          else
          {
            DataOut[i] = true_labels[l_n[s_index]];
            tt = 0;
            tmp_label = l_n[s_index];
            while(true_labels[tmp_label] != tmp_label)
            {
              tmp_label = true_labels[tmp_label];
            }
            while (l_index[tt] != 100)
            {

              tmp_label1 = l_n[l_index[tt]];
              while(true_labels[tmp_label1] != tmp_label1)
              {
                tmp_label1 = true_labels[tmp_label1];
              }

              if( tmp_label > tmp_label1)
                true_labels[tmp_label]=true_labels[tmp_label1];
              else
                true_labels[tmp_label1] = true_labels[tmp_label];

              tt +=1;
            } 
          }
 
        }/////here
      }
    }
  }
*/
  
  for(int ff=available_label;ff>=1;ff--)
  {
    loop_flag = true;
    while (loop_flag)
    {
      if( true_labels[ff] == true_labels[true_labels[ff]])
        loop_flag = false;
      else
        true_labels[ff] = true_labels[true_labels[ff]];
    }
  }
  tmp = 1;
  for (int ff=1;ff<=available_label;ff++)
  {
    if( true_labels[ff] == ff)
    {
      objects[ff] = tmp;
      tmp++;
    }
  }
  for(int i=0;i<no_pixels;i++)
  {
    if(DataOut[i] !=0)
      // Hussein 26/02/09
      DataOut[i] = objects[true_labels[DataOut[i]]];
  }

  return Data_Wrap_Struct( cMalloc, 0, deleteMallocObject, (void *)DataOut );
}

//***************************************** Main function for making Ruby extension ************************************
//**********************************************************************************************************************
extern "C" {
void Init_connected_components(void)
{
  rb_require( "hornetseye" );
  mHornetseye = rb_const_get( rb_cObject, rb_intern( "Hornetseye" ) );
  cMalloc = rb_const_get( mHornetseye, rb_intern( "Malloc" ) );
  cMultiArray = rb_const_get( mHornetseye, rb_intern( "Sequence_" ) );
  dMultiArray = rb_const_get( mHornetseye, rb_intern( "Sequence_" ) );
  rb_define_method( cMultiArray, "_labeling", RUBY_METHOD_FUNC( connected_component ), 4 );
  rb_define_method( cMultiArray, "_labeling_1", RUBY_METHOD_FUNC( connected_component_1 ), 4 );
  rb_define_method( cMultiArray, "_labeling_2", RUBY_METHOD_FUNC( connected_component_2 ), 8 );
  rb_define_method( dMultiArray, "_areas",RUBY_METHOD_FUNC(component_areas),4);
  rb_define_method( dMultiArray, "_centre_of_gravity",RUBY_METHOD_FUNC(component_centre_of_gravity),4);
  rb_require( "connected_components_ext.rb" );
}
}

