require 'hornetseye'
require 'nanomagnetics'

include Hornetseye

class Controller

  include DRbUndumped

  HOME = 0.0, 0.0, 20.0
  XYFINERESOLUTION = (262144/20.0)
  XYOFFSET = 0x20000
  ZFINERESOLUTION = 1/6.10352e-4
  ZOFFSET = 0x300

  def initialize
    @nanomagnetics = NanomagneticsDummy.new
    # @nanomagnetics = Nanomagnetics.new
    @x, @y, @z = 0.0, 0.0, 20.0
    @thread = nil
  end

  def pos
    [ @x, @y, @z ]
  end

  def home
    drive *HOME
  end

  def drive( x, y, z, n )
    @thread.join if @thread != nil
    x = -10 if x < -10
    x = 9.999 if x > 9.999
    y = -10 if y < -10
    y = 9.999 if y > 9.999
    z = -0.468 if z < -0.468
    z = 38.999 if z > 38.999
    n = 2 if n < 2
    ramp = Sequence.sfloat( n ).indgen! * ( 1.0 / ( n - 1 ) )
    xr, yr, zr =
      ( ( @x + ramp * ( x - @x ) ) * XYFINERESOLUTION + XYOFFSET ).to_uint,
      ( ( @y + ramp * ( y - @y ) ) * XYFINERESOLUTION + XYOFFSET ).to_uint,
      ( ( @z + ramp * ( z - @z ) ) * ZFINERESOLUTION  + ZOFFSET  ).to_usint
    @thread = Thread.new do
      @nanomagnetics.piezo xr, yr, zr
    end
    @x, @y, @z = x, y, z
  end

  def test( n )
    xa, ya, za =
      Sequence.uint( n ).fill!( HOME[0] ),
      Sequence.uint( n ).fill!( HOME[1] ),
      Sequence.usint( n ).fill!( HOME[2] )
    @nanomagnetics.piezo xa, ya, za
  end

end
