#!/usr/bin/env ruby
require 'Qt4'
require 'modeldialog'
require 'visionbase'
require 'ui_feedbackwindow'
require 'qrc_visiongui'
require 'drb'
require 'drb/acl'
require 'hornetseye'
include Hornetseye

Dir.glob( 'plugin_*.rb' ).each { |file| require file }

class Feedback

  include DRbUndumped

  attr_accessor :model

  attr_reader :pos

  def initialize( window )
    @model = nil
    @thread = nil
    @pos = nil
  end

  def stream( w, h, str )
    @thread.join if @thread != nil
    @pos = nil
    if @model != nil
      img = MultiArray( UBYTE, w, h ).new Memory.import( str ), 0
      @thread = Thread.new do
        if @model.tracking_algorithm != nil
          if @model.tracking
            @model.tracking_algorithm.update img
            @model.tracking = false unless @model.tracking_algorithm.success?
          end
          unless @model.tracking
            @model.recognition_algorithm.update img
            @model.tracking = true if @model.recognition_algorithm.success?
          end
        else
          @model.recognition_algorithm.update img
        end
      end
      if model.success?
        @pos = [ @model.pose[ 0, 2 ],
                 @model.pose[ 1, 2 ],
                 Math.atan2( model.pose[ 1, 0 ], model.pose[ 0, 0 ] ) ]
      end
    end
    @pos
  end

end

class FeedbackWindow < Qt::MainWindow

  STATUSDELAY = 5000

  slots 'conf_load()'
  slots 'toggle()'
  slots 'tray(QSystemTrayIcon::ActivationReason)'

  def initialize( parent = nil )
    super parent
    @ui = Ui::FeedbackWindow.new
    @ui.setupUi self
    tray_menu = Qt::Menu.new( self ) do |t|
      t.addAction @ui.action_toggle
      t.addSeparator
      t.addAction @ui.action_quit
    end
    @tray = Qt::SystemTrayIcon.new self
    @tray.contextMenu = tray_menu
    @tray.icon = windowIcon

    @vision = Feedback.new self

    connect @ui.action_toggle, SIGNAL('triggered()'), self, SLOT('toggle()')
    connect @ui.action_load_conf, SIGNAL('triggered()'),
      self, SLOT('conf_load()')
    connect @ui.action_close, SIGNAL('triggered()'), self, SLOT('close()')
    connect @ui.action_quit, SIGNAL('triggered()'), $qApp, SLOT('quit()')
    connect @tray, SIGNAL('activated(QSystemTrayIcon::ActivationReason)'),
      self, SLOT('tray(QSystemTrayIcon::ActivationReason)')
    @tray.show

    DRb.install_acl ACL.new( [ 'allow', 'localhost' ] )
    DRb.start_service "druby://:20000", @vision

    @timer = startTimer 40
  end

  def conf_load
    begin
      file = Qt::FileDialog.getOpenFileName self, 'Load Configuration',
                                            @conf_file,
                                            'Config Files (*.dat);;' +
                                            'All files (*)'
      if file
        statusBar.showMessage "Loading #{file}", STATUSDELAY
        load_conf file
      else
        statusBar.showMessage 'Aborted loading of configuration', STATUSDELAY
      end
    rescue Exception => e
      statusBar.showMessage e.to_s, STATUSDELAY
      Qt::MessageBox::critical self, 'Error Loading Configuration', e.to_s
    end
  end

  def load_conf( file )
    f = File.new file, 'rb'
    models = Marshal.load f.read
    # Force recognition in next step.
    models.collect { |m| m.tracking = false }
    raise "Parameter does not specify any object to recognise" if models.empty?
    if models.size == 1
      @vision.model = models.first
    else
      dialog = ModelDialog.new self
      dialog.model_names = models.collect { |m| m.name }
      if dialog.exec == Qt::Dialog::Accepted
        m = models.find { |m| m.name == dialog.model }
        @vision.model = models.find { |m| m.name == dialog.model }
      else
        statusBar.showMessage 'Aborted loading of configuration', STATUSDELAY
      end
    end
  end

  def toggle
    if visible?
      hide
    else
      show
    end
  end

  def tray( reason )
    toggle if reason == Qt::SystemTrayIcon::Trigger
  end

  def hideEvent( e )
    @ui.action_toggle.text = '&Restore'
    e.accept
  end

  def showEvent( e )
    @ui.action_toggle.text = '&Minimise'
    e.accept
  end

  def closeEvent( e )
    if @tray.visible?
      message = <<EOS
The application will continue to run in the system tray.
To terminate the program right-click on the tray icon and
choose Quit in the context menu.
EOS
      @tray.showMessage windowTitle, message, Qt::SystemTrayIcon::Information,
        5000
      hide
      e.ignore
    end
  end

  def timerEvent( e )
    DRb.thread.join 0.04
    pos = @vision.pos
    if pos != nil
      @ui.x_label.text = "%5.1f" % pos[0]
      @ui.y_label.text = "%5.1f" % pos[1]
    else
      @ui.x_label.text = ''
      @ui.y_label.text = ''
    end
  end

end

app = Qt::Application.new ARGV
unless Qt::SystemTrayIcon.isSystemTrayAvailable
  Qt::MessageBox.critical nil, 'System Tray Required',
                          'Could not detect a system tray on this desktop'
end
window = FeedbackWindow.new
app.exec
