// #include <iostream>
#include <boost/smart_ptr.hpp>
#include <errno.h>
#include <fcntl.h>
#include <ruby.h>
#ifdef read
#undef read
#endif
#ifdef write
#undef write
#endif
#ifndef CBW
#include <pci-dio24.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#else
#include "cbw.h"
#endif
#include "error.hh"

const int x = 0;
const int
  Bit0 = 0x1,
  Bit1 = 0x2,
  Bit2 = 0x4,
  Bit3 = 0x8,
  Bit4 = 0x10,
  Bit5 = 0x20,
  Bit6 = 0x40,
  Bit7 = 0x80,
  Bit8 = 0x100,
  Bit9 = 0x200,
  Bit10 = 0x400,
  Bit11 = 0x800,
  Bit12 = 0x1000,
  Bit13 = 0x2000,
  Bit14 = 0x4000,
  Bit15 = 0x8000;

const char
  Fiber_L = Bit6 | Bit3,
  Fiber_CS    = Bit6 | Bit3 | Bit0,
  Slider_L    = Bit6 | Bit3 | Bit1,         /* now slider_cont byte */
  Slider_CS   = Bit6 | Bit3 | Bit1 | Bit0,
  Slider_CONT = Bit6 | Bit3 | Bit2,
  Slider_JOY  = Bit6 | Bit7 | Bit3 | Bit2 | Bit0,
  Cont_stat   = Bit6 | Bit3 | Bit2 | Bit1,
  Photo_stat  = Bit6 | Bit3 | Bit2 | Bit1 | Bit0,
/* BANK B */
  Vx_H_L      = Bit6 | Bit4,
  Vx_H_CS     = Bit6 | Bit4 | Bit0,
  Vx_L_L      = Bit6 | Bit4 | Bit1,
  Vx_L_CS     = Bit6 | Bit4 | Bit1 | Bit0,
  Vy_H_L      = Bit6 | Bit4 | Bit2,
  Vy_H_CS     = Bit6 | Bit4 | Bit2 | Bit0,
  Vy_L_L      = Bit6 | Bit4 | Bit2 | Bit1,
  Vy_L_CS     = Bit6 | Bit4 | Bit2 | Bit1 | Bit0,
/* BANK C */
  Vs_L        = Bit6 ,
  Vs_CS       = Bit6 | Bit0,
  Vzoff_L     = Bit6 | Bit1,
  Vzoff_CS    = Bit6 | Bit1 | Bit0,
  Iset_L      = Bit6 | Bit2,
  Iset_CS     = Bit6 | Bit2 | Bit0,
  DacCardRegister     = Bit6 | Bit2 | Bit1,
  SPARE1_CS   = Bit6 | Bit2 | Bit1 | Bit0,

/* BANK D */
  AD_STAT                = Bit5 | Bit3,
  HallCurrent_L          = Bit5 | Bit3 | Bit0,
  HallCurrent_CS         = Bit5 | Bit3 | Bit1,
  HallOffset_L           = Bit5 | Bit3 | Bit1 | Bit0,
  HallOffset_CS          = Bit5 | Bit3 | Bit2,
  HallStatusRegister = Bit5 | Bit3 | Bit2 | Bit0,
  empty6	           = Bit5 | Bit3 | Bit2 | Bit1,
  empty7	           = Bit5 | Bit3 | Bit2 | Bit1 | Bit0,
/* BANK E */
  AD_DATA	= Bit7 | Bit5 ,
  AD_BUSY	= Bit7 | Bit5 | Bit0,
  AD_CONVERT	= Bit7 | Bit5 | Bit1,
  AD2_DATA    = Bit7 | Bit5 | Bit1 | Bit0,
  empty10	= Bit7 | Bit5 | Bit2,
  empty11	= Bit7 | Bit5 | Bit2 | Bit0,
  empty12	= Bit7 | Bit5 | Bit2 | Bit1,
  empty13	= Bit7 | Bit5 | Bit2 | Bit1 | Bit0,
  
/* BANK F */
  READ_DC_JOYSTICK = Bit7 | Bit3 | Bit4,
  Read_limit_Switches = Bit7 | Bit3 | Bit4 | Bit0,
  Read_index_counter = Bit7 | Bit3 | Bit4 | Bit1,
  Read_Counter= Bit7 | Bit3 | Bit4 | Bit1 | Bit0,
  empty14	= Bit7 | Bit3 | Bit4 | Bit3 | Bit4 | Bit2,
  empty15	= Bit7 | Bit3 | Bit4 | Bit2 | Bit0,
  empty16	= Bit7 | Bit3 | Bit4 | Bit2 | Bit1,
  empty17	= Bit7 | Bit3 | Bit4 | Bit2 | Bit1 | Bit0,
/* BANK G */
    Load_Counter_register = Bit3 | Bit4,
  Load_index_Counter_Register = Bit3 | Bit4 | Bit0,
  Load_registers_to_counters = Bit3 | Bit4 | Bit1,
  Load_DC_status_register = Bit3 | Bit4 | Bit1 | Bit0,
  empty18	= Bit3 | Bit4 | Bit3 | Bit4 | Bit2,
  empty19	= Bit3 | Bit4 | Bit2 | Bit0,
  empty20	= Bit3 | Bit4 | Bit2 | Bit1,
  empty21	= Bit3 | Bit4 | Bit2 | Bit1 | Bit0,
  
/* BANK H */
  Photo_offset_dac_L = Bit5 | Bit6 | Bit3 | Bit4 ,
  Photo_offset_DAC_CS = Bit5 | Bit6 | Bit3 | Bit4 | Bit0,
/* FOR BOBJI's 2nd Interferometer card ONLY*/
  Photo_offset_dac_L2 = Bit5 | Bit6 | Bit3 | Bit4 | Bit1,
  Photo_offset_DAC_CS2 = Bit5 | Bit6 | Bit3 | Bit4 | Bit1 | Bit0,
    Photo_stat2	= Bit5 | Bit6 | Bit3 | Bit4 | Bit2,
/*****************************************************************/
  empty25	= Bit5 | Bit6 | Bit3 | Bit4 | Bit2 | Bit0,
  empty26	= Bit5 | Bit6 | Bit3 | Bit4 | Bit2 | Bit1,
  empty27	= Bit5 | Bit6 | Bit3 | Bit4 | Bit2 | Bit1 | Bit0;

 /* Steve's Slider   5/5/1998    */

const short int
  Steve_n      = Bit12 | Bit3,
  Steve_s      = Bit12 | Bit6 | Bit8,
  Steve_e      = Bit12 | Bit6 | Bit3,
  Steve_w      = Bit12 | Bit8,
  Steve_u      = Bit12 | Bit7,
  Steve_d      = Bit12 | Bit10 ,

  Steve_Fiber_n      = Bit12 | Bit4,
  Steve_Fiber_s      = Bit12 | Bit0 | Bit1,
  Steve_Fiber_e      = Bit12 | Bit0,
  Steve_Fiber_w      = Bit12 | Bit1 | Bit4,
  Steve_Fiber_u      = Bit12 | Bit9,
  Steve_Fiber_d      = Bit12 | Bit5,

/* Bit2 & Bit12 controls the HV amp / Slider pulse change        */
/* This is x-y-phi slider, Bits 4/5/8/9   25/3/98 */
  Fiber_F     = Bit12 | Bit1 | Bit8,
  Fiber_B     = Bit12 | Bit4 | Bit11 | Bit0 | Bit3 | Bit5,

  Fiber_E     = Bit12 | Bit1 | Bit4 | Bit8,
  Fiber_W     = Bit12 | Bit11 | Bit0 | Bit3 | Bit5,

  phi_minus    = Bit12 | Bit1,
  phi_plus     = Bit12 | Bit4 | Bit8 | Bit11 | Bit0 | Bit3 | Bit5,

/* This is z-x-theta slider, Bits 0/1/4/10   */
  Fiber_N     = Bit12 | Bit0 | Bit5 | Bit11 | Bit1 | Bit4 | Bit8,
  Fiber_S     = Bit12 | Bit3,

  Fiber_Z_F    = Bit12 | Bit0 | Bit5,
  Fiber_Z_B    = Bit12 | Bit3 | Bit11 | Bit1 | Bit4 | Bit8,

  Theta_plus   = Bit12 | Bit0 | Bit3 | Bit11 | Bit1 | Bit4 | Bit8,
  Theta_minus  = Bit12 | Bit5,

/* new ones with 4 connections to Sample slider...  done on 25/3/98 */
  Slid_F    = Bit12 | Bit6 | Bit10,
  Slid_B    = Bit12 | Bit7 | Bit9,

  Slid_R    = Bit12 | Bit6 | Bit7,
  Slid_L    = Bit12 | Bit9 | Bit10,

  Slid_CCW  = Bit12 | Bit6,
  Slid_CW   = Bit12 | Bit7 | Bit9 | Bit10,

/* MODIFIED FOR NITEM4 BY MSB on 28/06/07 */
  LT_slider_up_word  =   Bit12 | Bit4,
  LT_slider_down_word=   Bit12 | Bit11,
  LT_slider_North_word  =   Bit12 | Bit10,
  LT_slider_South_word  =   Bit12 | Bit7,
  LT_slider_East_word   =   Bit12 | Bit8,
  LT_slider_West_word   =   Bit12 | Bit6,
  LT_slider_Clockwise_word = Bit12 | Bit5,
  LT_slider_Counterclockwise_word = Bit12 | Bit9,
/*end of modifications*/


  LT_slider_test_word   =   Bit12 | Bit5 | Bit4 | Bit0 | Bit1 | Bit3 |
                            Bit11 | Bit8 | Bit6 | Bit10 | Bit7 | Bit9,

  Short_piezos = Bit8 | Bit11 | Bit6 | Bit7 | Bit9 | Bit10,

  HV_TO_PZT = Bit2 | Bit4;

#define VALUEFUNC(x) ((VALUE (*)(ANYARGS))x)

class Nanomagnetics
{
public:
#ifndef CBW
  Nanomagnetics( const std::string &deviceA,
		 const std::string &deviceB,
		 const std::string &deviceC ) throw (hornetseye::Error);
#else
  Nanomagnetics( int boardId ) throw (hornetseye::Error);
#endif
  virtual ~Nanomagnetics(void);
  // void test(void) throw (hornetseye::Error);
  void close(void);
  void dacSlider( unsigned short int data ) throw (hornetseye::Error);
  void wakeupSlider( short int slider ) throw (hornetseye::Error);
  void sleepSlider(void) throw (hornetseye::Error);
  void outX( unsigned int x ) throw (hornetseye::Error);
  void outY( unsigned int y ) throw (hornetseye::Error);
  void dacZOff( unsigned short int z ) throw (hornetseye::Error);
  void slider( int samples,
               unsigned short int *data ) throw (hornetseye::Error);
  void piezo( int samples,
              unsigned int *dataX,
              unsigned int *dataY,
              unsigned short int *dataZ ) throw (hornetseye::Error);
  static VALUE cRubyClass;
  static VALUE registerRubyClass(void);
  static void deleteRubyObject( void *ptr );
#ifndef CBW
  static VALUE wrapNew( VALUE rbClass,
                        VALUE rbDeviceA, VALUE rbDeviceB, VALUE rbDeviceC );
#else
  static VALUE wrapNew( VALUE rbClass, VALUE rbBoardId );
#endif
  static VALUE wrapClose( VALUE rbSelf );
  static VALUE wrapWakeupSlider( VALUE rbSelf, VALUE rbSlider );
  static VALUE wrapSleepSlider( VALUE rbSelf );
  static VALUE wrapSlider( VALUE rbSelf, VALUE rbSamples,
                           VALUE rbData );
  static VALUE wrapPiezo( VALUE rbSelf, VALUE rbSamples,
                          VALUE rbDataX, VALUE rbDataY, VALUE rbDataZ );
  static VALUE wrapOutX( VALUE rbSelf, VALUE rbX );
  static VALUE wrapOutY( VALUE rbSelf, VALUE rbY );
  static VALUE wrapDacZOff( VALUE rbSelf, VALUE rbZ );
protected:
  static const unsigned long delay;
  static const unsigned char AD_STAT;
  static const unsigned char AD_DATA;
  static const unsigned char HALL_STATUS_REGISTER;
  static const unsigned char DAC_CARD_REGISTER;
  void writeData( unsigned char address, unsigned short int data )
    throw (hornetseye::Error);
  void writeDataX( unsigned char address1,
		   unsigned char address2,
		   unsigned short int data ) throw (hornetseye::Error);
  // unsigned short int readData( unsigned char address )
  //   throw (hornetseye::Error);
#ifndef CBW
  std::string m_deviceA;
  std::string m_deviceB;
  std::string m_deviceC;
  int m_fdA;
  int m_fdB;
  int m_fdC;
#else
  int m_boardId;
#endif
  /* unsigned short hallStatus;
  int channelMap[8];
  std::map< int, int > gainMap; */
};

typedef boost::shared_ptr< Nanomagnetics > NanomagneticsPtr;

using namespace hornetseye;
using namespace std;

const unsigned long Nanomagnetics::delay = 100000;
const unsigned char Nanomagnetics::AD_STAT = 0x28;
const unsigned char Nanomagnetics::AD_DATA = 0xA0;
const unsigned char Nanomagnetics::HALL_STATUS_REGISTER = 0x2D;
const unsigned char Nanomagnetics::DAC_CARD_REGISTER = 0x46;

#ifdef CBW
string cbError( int errCode )
{
  char errMsg[ERRSTRLEN];
  cbGetErrMsg( errCode, errMsg );
  return string( errMsg );
}

#define CBWERRORMACRO( condition, class, params, message ) \
{                                                          \
  int _n = condition;                                      \
  if ( _n != 0 ) {                                         \
    char _msg[ERRSTRLEN];                                  \
    class _e params;                                       \
    _e << message << ": " << _msg;                         \
    throw _e;                                              \
  }                                                        \
}
#endif

VALUE Nanomagnetics::cRubyClass = Qnil;

#ifndef CBW
Nanomagnetics::Nanomagnetics( const std::string &deviceA = "/dev/dio24/dio0_0A",
			      const std::string &deviceB = "/dev/dio24/dio0_0B",
			      const std::string &deviceC = "/dev/dio24/dio0_0C" )
  throw (Error):
  m_deviceA(deviceA), m_deviceB(deviceB), m_deviceC(deviceC),
  m_fdA(-1), m_fdB(-1), m_fdC(-1)
#else
Nanomagnetics::Nanomagnetics( int boardId ) throw (Error):
  m_boardId(boardId)
#endif
{
  try {
#ifndef CBW
    // 1)
    int errsv;
    m_fdA = open( m_deviceA.c_str(), O_RDWR ); errsv = errno;
    ERRORMACRO( m_fdA != -1, Error, ,
		"Error opening device \"" << m_deviceA << "\": "
		<< strerror( errsv ) );
    m_fdB = open( m_deviceB.c_str(), O_RDWR ); errsv = errno;
    ERRORMACRO( m_fdB != -1, Error, ,
		"Error opening device \"" << m_deviceB << "\": "
		<< strerror( errsv ) );
    m_fdC = open( m_deviceC.c_str(), O_RDWR ); errsv = errno;
    ERRORMACRO( m_fdC != -1, Error, ,
		"Error opening device \"" << m_deviceC << "\": "
		<< strerror( errsv ) );

    usleep( delay );
    ioctl( m_fdA, DIO_SET_DIRECTION, PORT_OUTPUT );// 3)
    usleep( delay );
    ioctl( m_fdB, DIO_SET_DIRECTION, PORT_OUTPUT );
    usleep( delay );
    ioctl( m_fdC, DIO_SET_DIRECTION, LOW_PORT_OUTPUT );
    usleep( delay );
    ioctl( m_fdC, DIO_SET_DIRECTION, HIGH_PORT_OUTPUT );
    usleep( delay );
#else
    CBWERRORMACRO( cbDConfigPort( m_boardId, FIRSTPORTA, DIGITALOUT ),
                   Error, , "Error accessing port A of board " << m_boardId );
    CBWERRORMACRO( cbDConfigPort( m_boardId, FIRSTPORTB, DIGITALOUT ),
                   Error, , "Error accessing port B of board " << m_boardId );
    CBWERRORMACRO( cbDConfigPort( m_boardId, FIRSTPORTCL, DIGITALOUT ),
                   Error, , "Error accessing port CL of board " << m_boardId );
    CBWERRORMACRO( cbDConfigPort( m_boardId, FIRSTPORTCH, DIGITALOUT ),
                   Error, , "Error accessing port CH of board " << m_boardId );
#endif
  } catch ( Error &e ) {
#ifndef CBW
    if ( m_fdC != -1 ) ::close( m_fdC );
    if ( m_fdB != -1 ) ::close( m_fdB );
    if ( m_fdA != -1 ) ::close( m_fdA );
#endif
    throw e;
  };
}

Nanomagnetics::~Nanomagnetics(void)
{
  close();
}

void Nanomagnetics::close(void)
{
#ifndef CBW
  if ( m_fdA != -1 ) {
    ::close( m_fdA );
    m_fdA = -1;
  };
  if ( m_fdB != -1 ) {
    ::close( m_fdB );
    m_fdB = -1;
  };
  if ( m_fdC != -1 ) {
    ::close( m_fdC );
    m_fdC = -1;
  };
#endif
}

void Nanomagnetics::dacSlider( unsigned short int data )  throw (Error)
{
  writeDataX( Slider_CS, Fiber_L, data );
}

void Nanomagnetics::wakeupSlider( short int slider  ) throw (Error)
{
  // cerr << "Wake up slider" << endl;
  dacSlider(0);
  writeData( Slider_L, Bit5 );
  writeData( Slider_CONT, slider );
}

void Nanomagnetics::sleepSlider(void) throw (Error)
{
  // cerr << "Sleep slider" << endl;
  dacSlider(0);
  writeData( Slider_CONT, Bit12 );
  writeData( Slider_L, 0 );
}

void Nanomagnetics::outX( unsigned int x ) throw (Error)
{
  // 18 bits
  unsigned short int
    x_l = ( x << 2 ) & 0x03FF,
    x_h = ( x >> 8 ) << 6;
  writeData( Vx_H_CS, x_h );
  // writeDataX( Vx_L_CS, Vx_L_L, x_l );// or Vx_H_L?
  writeDataX( Vx_L_CS, Vx_H_L, x_l );// not Vx_L_L any more?
}

void Nanomagnetics::outY( unsigned int y ) throw (Error)
{
  // 18 bits
  unsigned short int
    y_l = ( y << 2 ) & 0x03FF,
    y_h = ( y >> 8 ) << 6;
  writeData( Vy_H_CS, y_h );
  writeDataX( Vy_L_CS, Vy_L_L, y_l );
}

void Nanomagnetics::dacZOff( unsigned short int z )  throw (Error)
{
  writeDataX( Vzoff_CS, Vzoff_L, z );
}

void Nanomagnetics::writeData( unsigned char address,
			       unsigned short int data ) throw (Error)
{
#ifndef CBW
  const unsigned char zero = 0;
  write( m_fdC, &zero, 1 );// !!!
  // usleep( delay );
  // ioctl( m_fdA, DIO_SET_DIRECTION, PORT_OUTPUT );
  // usleep( delay );
  // ioctl( m_fdB, DIO_SET_DIRECTION, PORT_OUTPUT );
  // usleep( delay );
  unsigned char dataLow = (unsigned char)( data &0x00FF );
  unsigned char dataHigh = (unsigned char)( ( data &0xFF00 ) >> 8 );
  write( m_fdA, &dataLow, 1 );
  write( m_fdB, &dataHigh, 1 );
  write( m_fdC, &address, 1 );
  write( m_fdC, &zero, 1 );
#else
  cbDOut( m_boardId, FIRSTPORTCL, 0 );// !!!
  cbDOut( m_boardId, FIRSTPORTCH, 0 );
  cbDConfigPort( m_boardId, FIRSTPORTA, DIGITALOUT );// !!!
  cbDConfigPort( m_boardId, FIRSTPORTB, DIGITALOUT );
  unsigned char dataLow = (unsigned char)( data &0x00FF );
  unsigned char dataHigh = (unsigned char)( ( data &0xFF00 ) >> 8 );
  cbDOut( m_boardId, FIRSTPORTA, dataLow );
  cbDOut( m_boardId, FIRSTPORTB, dataHigh );
  cbDOut( m_boardId, FIRSTPORTCL, address & 0xF );
  cbDOut( m_boardId, FIRSTPORTCH, ( address & 0xF0 ) >> 4 );
  cbDOut( m_boardId, FIRSTPORTCL, 0 );// !!!
  cbDOut( m_boardId, FIRSTPORTCH, 0 );
#endif
}

void Nanomagnetics::writeDataX( unsigned char address1,
				unsigned char address2,
				unsigned short int data ) throw (Error)
{
#ifndef CBW
  const unsigned char zero = 0;
  unsigned char dataLow = (unsigned char)( data &0x00FF );
  unsigned char dataHigh = (unsigned char)( ( data &0xFF00 ) >> 8 );
  write( m_fdA, &dataLow, 1 );
  write( m_fdB, &dataHigh, 1 );
  write( m_fdC, &address1, 1 );
  write( m_fdC, &address2, 1 );
  write( m_fdC, &zero, 1 );
#else
  unsigned char dataLow = (unsigned char)( data &0x00FF );
  unsigned char dataHigh = (unsigned char)( ( data &0xFF00 ) >> 8 );
  cbDOut( m_boardId, FIRSTPORTA, dataLow );
  cbDOut( m_boardId, FIRSTPORTB, dataHigh );
  cbDOut( m_boardId, FIRSTPORTCL, address1 & 0xF );
  cbDOut( m_boardId, FIRSTPORTCH, ( address1 & 0xF0 ) >> 4 );
  cbDOut( m_boardId, FIRSTPORTCL, address2 & 0xF );
  cbDOut( m_boardId, FIRSTPORTCH,    // outY( dataY[i] );
 ( address2 & 0xF0 ) >> 4 );
  cbDOut( m_boardId, FIRSTPORTCL, 0 );
  cbDOut( m_boardId, FIRSTPORTCH, 0 );
#endif
}

// unsigned short int Nanomagnetics::readData( unsigned char address )
//   throw (Error)
// {
// #ifndef CBW
//   const unsigned char zero = 0;
//   write( m_fdC, &zero, 1 );
//   ioctl( m_fdA, DIO_SET_DIRECTION, PORT_INPUT );
//   usleep( delay );
//   ioctl( m_fdB, DIO_SET_DIRECTION, PORT_INPUT );
//   usleep( delay );
//   write( m_fdC, &address, 1 );
//   unsigned char dataLow;  read( m_fdA, &dataLow, 1 ); usleep( delay );
//   unsigned char dataHigh; read( m_fdB, &dataHigh, 1 ); usleep( delay );
//   unsigned short int retval =
//     ( ((unsigned short int)dataHigh) << 8 ) | dataLow;
//   write( m_fdC, &zero, 1 );
//   return retval;
// #else
//   CBWERRORMACRO( cbDConfigPort( m_boardId, FIRSTPORTA, DIGITALOUT ),
//                  Error, , "Error accessing port A of board " << m_boardId );
//   CBWERRORMACRO( cbDConfigPort( m_boardId, FIRSTPORTB, DIGITALOUT ),
//                  Error, , "Error accessing port B of board " << m_boardId );
//   // ... !!!
//   unsigned short int retval = 0;
//   return retval;
// #endif
// }

void Nanomagnetics::slider( int samples,
                            unsigned short int *data )
 throw (hornetseye::Error)
{
  // Call wakeupSLider before!!!
  for ( int i=0; i<samples; i++ ) {
    // Insert delay???
    dacSlider( data[i] );
  };
  // Call sleepSlider afterwards!!!
}

void Nanomagnetics::piezo( int samples,
                           unsigned int *dataX,
                           unsigned int *dataY,
                           unsigned short int *dataZ )
 throw (hornetseye::Error)
{
  for ( int i=0; i<samples; i++ ) {
    outX( dataX[i] );
    outY( dataY[i] );
    dacZOff( dataZ[i] );
  };
}

VALUE Nanomagnetics::registerRubyClass(void)
{
  cRubyClass = rb_define_class( "Nanomagnetics", rb_cObject );
#ifndef CBW
  rb_define_singleton_method( cRubyClass, "new", VALUEFUNC( wrapNew ), 3 );
#else
  rb_define_singleton_method( cRubyClass, "new", VALUEFUNC( wrapNew ), 1 );
#endif
  rb_define_method( cRubyClass, "wakeup_slider", VALUEFUNC( wrapWakeupSlider ), 1 );
  rb_define_method( cRubyClass, "sleep_slider", VALUEFUNC( wrapSleepSlider ), 0 );
  rb_define_method( cRubyClass, "slider", VALUEFUNC( wrapSlider ), 2 );
  rb_define_method( cRubyClass, "piezo", VALUEFUNC( wrapPiezo ), 4 );
  rb_define_method( cRubyClass, "close", VALUEFUNC( wrapClose ), 0 );
  rb_define_method( cRubyClass, "out_x", VALUEFUNC( wrapOutX ), 1 );
  rb_define_method( cRubyClass, "out_y", VALUEFUNC( wrapOutY ), 1 );
  rb_define_method( cRubyClass, "dac_zoff", VALUEFUNC( wrapDacZOff ), 1 );
  rb_define_const( cRubyClass, "SLIDER_UP", INT2FIX(LT_slider_up_word) );
  rb_define_const( cRubyClass, "SLIDER_DOWN", INT2FIX(LT_slider_down_word) );
  rb_define_const( cRubyClass, "SLIDER_NORTH", INT2FIX(LT_slider_North_word) );
  rb_define_const( cRubyClass, "SLIDER_SOUTH", INT2FIX(LT_slider_South_word) );
  rb_define_const( cRubyClass, "SLIDER_EAST", INT2FIX(LT_slider_East_word) );
  rb_define_const( cRubyClass, "SLIDER_WEST", INT2FIX(LT_slider_West_word) );
  rb_define_const( cRubyClass, "SLIDER_CLOCKWISE", INT2FIX(LT_slider_Clockwise_word) );
  rb_define_const( cRubyClass, "SLIDER_COUNTERCLOCKWISE", INT2FIX(LT_slider_Counterclockwise_word) );
  return cRubyClass;
}

void Nanomagnetics::deleteRubyObject( void *ptr )
{
  delete (NanomagneticsPtr *)ptr;
}

#ifndef CBW
VALUE Nanomagnetics::wrapNew( VALUE rbClass, VALUE rbDeviceA, VALUE rbDeviceB,
                              VALUE rbDeviceC )
{
  VALUE retVal = Qnil;
  try {
    rb_check_type( rbDeviceA, T_STRING );
    rb_check_type( rbDeviceB, T_STRING );
    rb_check_type( rbDeviceC, T_STRING );
    NanomagneticsPtr ptr
      ( new Nanomagnetics
        ( StringValuePtr( rbDeviceA ), StringValuePtr( rbDeviceB ),
          StringValuePtr( rbDeviceC ) ) );
    retVal = Data_Wrap_Struct( rbClass, 0, deleteRubyObject,
                               new NanomagneticsPtr( ptr ) );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  };
  return retVal;
}
#else
VALUE Nanomagnetics::wrapNew( VALUE rbClass, VALUE rbBoardId )
{
  VALUE retVal = Qnil;
  try {
    NanomagneticsPtr ptr
      ( new Nanomagnetics
        ( NUM2INT( rbBoardId ) ) );
    retVal = Data_Wrap_Struct( rbClass, 0, deleteRubyObject,
                               new NanomagneticsPtr( ptr ) );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  };
  return retVal;
}
#endif

VALUE Nanomagnetics::wrapClose( VALUE rbSelf )
{
  NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
  (*self)->close();
  return rbSelf;
}

VALUE Nanomagnetics::wrapWakeupSlider( VALUE rbSelf, VALUE rbSlider )
{
  try {
    NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
    (*self)->wakeupSlider( NUM2INT( rbSlider ) );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  }
  return rbSelf;
}

VALUE Nanomagnetics::wrapSleepSlider( VALUE rbSelf )
{
  try {
    NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
    (*self)->sleepSlider();
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  }
  return rbSelf;
}

VALUE Nanomagnetics::wrapSlider( VALUE rbSelf, VALUE rbSamples,
                                 VALUE rbData )
{
  try {
    NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
    unsigned short int *p;
    Data_Get_Struct( rbData, unsigned short int, p );
    (*self)->slider( NUM2INT(rbSamples), p );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  };
  return rbSelf;
}

VALUE Nanomagnetics::wrapPiezo( VALUE rbSelf, VALUE rbSamples,
                                VALUE rbDataX, VALUE rbDataY, VALUE rbDataZ )
{
  try {
    NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
    unsigned int *pX, *pY;
    unsigned short int *pZ;
    Data_Get_Struct( rbDataX, unsigned int, pX );
    Data_Get_Struct( rbDataY, unsigned int, pY );
    Data_Get_Struct( rbDataZ, unsigned short int, pZ );
    (*self)->piezo( NUM2INT(rbSamples), pX, pY, pZ );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  };
  return rbSelf;
}

VALUE Nanomagnetics::wrapOutX( VALUE rbSelf, VALUE rbX )
{
  try {
    NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
    (*self)->outX( NUM2INT(rbX) );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  };
  return rbSelf;
}

VALUE Nanomagnetics::wrapOutY( VALUE rbSelf, VALUE rbY )
{
  try {
    NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
    (*self)->outY( NUM2INT(rbY) );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  };
  return rbSelf;
}

VALUE Nanomagnetics::wrapDacZOff( VALUE rbSelf, VALUE rbZ )
{
  try {
    NanomagneticsPtr *self; Data_Get_Struct( rbSelf, NanomagneticsPtr, self );
    (*self)->dacZOff( NUM2INT(rbZ) );
  } catch ( exception &e ) {
    rb_raise( rb_eRuntimeError, e.what() );
  };
  return rbSelf;
}

extern "C" {

  void Init_nanomagnetics(void)
  {
    rb_require( "hornetseye" );
    Nanomagnetics::registerRubyClass();
    rb_require( "nanomagnetics_ext.rb" );
  }

}
