require 'drb'

class Nanomagnetics

  class << self

    alias_method :orig_new, :new

    def new( deviceA = "/dev/dio24/dio0_0A", deviceB = "/dev/dio24/dio0_0B",
             deviceC = "/dev/dio24/dio0_0C" )
      orig_new( deviceA, deviceB, deviceC )
    end
    #def new( deviceId = 0 )
    #  orig_new( deviceId )
    #end

  end

  alias_method :orig_slider, :slider

  def slider( samples )
    if samples.typecode != Sequence::USINT
      raise "Samples for slider must be of type USINT"
    end
    orig_slider( samples.size, samples.data )
  end

  alias_method :orig_piezo, :piezo

  def piezo( samples_x, samples_y, samples_z )
    if ( samples_x.size != samples_y.size ) or
       ( samples_x.size != samples_z.size )
      raise "Sample arrays for x, y, and z must have same size"
    end
    if samples_x.typecode != Sequence::UINT
      raise "x-samples for piezo must be of type UINT"
    end
    if samples_y.typecode != Sequence::UINT
      raise "y-samples for piezo must be of type UINT"
    end
    if samples_z.typecode != Sequence::USINT
      raise "z-samples for piezo must be of type USINT"
    end
    orig_piezo( samples_x.size,
                samples_x.data, samples_y.data, samples_z.data )
  end

end

class NanomagneticsDummy

  def initialize( deviceA = "/dev/dio24_0A", deviceB = "/dev/dio24_0B",
                  deviceC = "/dev/dio24_0C" )
  end

  # def new( deviceId = 0 )
  # end

  def slider( samples )
    puts "slider( #{samples.inspect} )"
    if samples.typecode != Sequence::USINT
      raise "Samples for slider must be of type USINT"
    end
    sleep samples.size / 56692.0
  end

  def piezo( samples_x, samples_y, samples_z )
    puts "piezo( #{samples_x.inspect}, #{samples_y.inspect}, " +
         "#{samples_z.inspect} )"
    if ( samples_x.size != samples_y.size ) or
       ( samples_x.size != samples_z.size )
      raise "Sample arrays for x, y, and z must have same size"
    end
    if samples_x.typecode != Sequence::UINT
      raise "x-samples for piezo must be of type UINT"
    end
    if samples_y.typecode != Sequence::UINT
      raise "y-samples for piezo must be of type UINT"
    end
    if samples_z.typecode != Sequence::USINT
      raise "z-samples for piezo must be of type USINT"
    end
    sleep samples_x.size / 10814.0
  end

  def wakeup_slider( slider )
    puts "wakeup_slider( #{slider.inspect} )"
  end

  def sleep_slider
    puts "sleep_slider"
  end

end

