#!/usr/bin/env ruby
require 'Qt4'
require 'ui_serverwindow'
require 'qrc_visiongui'
require 'drb'
require 'drb/acl'
require 'controller'
require 'nanomagnetics'

class ServerWindow < Qt::MainWindow

  slots 'toggle()'
  slots 'tray(QSystemTrayIcon::ActivationReason)'

  def initialize( parent = nil )
    super parent
    @ui = Ui::ServerWindow.new
    @ui.setupUi self
    tray_menu = Qt::Menu.new( self ) do |t|
      t.addAction @ui.action_toggle
      t.addSeparator
      t.addAction @ui.action_quit
    end
    @tray = Qt::SystemTrayIcon.new self
    @tray.contextMenu = tray_menu
    @tray.icon = windowIcon

    @controller = Controller.new

    connect @ui.action_toggle, SIGNAL('triggered()'), self, SLOT('toggle()')
    connect @ui.action_close, SIGNAL('triggered()'), self, SLOT('close()')
    connect @ui.action_quit, SIGNAL('triggered()'), $qApp, SLOT('quit()')
    connect @tray, SIGNAL('activated(QSystemTrayIcon::ActivationReason)'),
      self, SLOT('tray(QSystemTrayIcon::ActivationReason)')
    @tray.show

    DRb.install_acl ACL.new( [ 'allow', 'localhost' ] )
    DRb.start_service "druby://:10000", @controller
    
    @timer = startTimer 40
  end

  def toggle
    if visible?
      hide
    else
      show
    end
  end

  def tray( reason )
    toggle if reason == Qt::SystemTrayIcon::Trigger
  end

  def hideEvent( e )
    @ui.action_toggle.text = '&Restore'
    e.accept
  end

  def showEvent( e )
    @ui.action_toggle.text = '&Minimise'
    e.accept
  end

  def closeEvent( e )
    if @tray.visible?
      message = <<EOS
The application will continue to run in the system tray.
To terminate the program right-click on the tray icon and
choose Quit in the context menu.
EOS
      @tray.showMessage windowTitle, message, Qt::SystemTrayIcon::Information,
        5000
      hide
      e.ignore
    end
  end

  def timerEvent( e )
    DRb.thread.join 0.04
    pos = @controller.pos
    @ui.x_label.text = "%6.3f V" % pos[0]
    @ui.y_label.text = "%6.3f V" % pos[1]
    @ui.z_label.text = "%6.3f V" % pos[2]
  end

end

app = Qt::Application.new ARGV
unless Qt::SystemTrayIcon.isSystemTrayAvailable
  Qt::MessageBox.critical nil, 'System Tray Required',
                          'Could not detect a system tray on this desktop'
end
window = ServerWindow.new
app.exec
