require 'Qt4'
require 'qrc_visiongui'
require 'ui_visiondialog'
require 'hornetseye'

class VisionDialog < Qt::Dialog

  slots 'select_model(int)'
  slots 'add()'
  slots 'delete()'
  slots 'model_name(QString)'
  slots 'add_tracking()'
  slots 'delete_tracking()'
  slots 'model_coord()'

  def initialize( parent, recognition_algorithms, tracking_algorithms )
    super parent

    # Setup dialog.
    @ui = Ui::VisionDialog.new
    @ui.setupUi self
    @coord_widget = CoordDisplay.new
    @ui.scroll_area.widget = @coord_widget
    @recognition_algorithms = recognition_algorithms
    @tracking_algorithms = tracking_algorithms
    @models = []

    # Populate controls.
    @recognition_widgets = []
    recognition_algorithms.each do |algorithm|
      @ui.recognition_combo.addItem algorithm.description
      widget = algorithm.config_widget
      @recognition_widgets.push widget
      @ui.recognition_stack.addWidget widget
    end
    @tracking_widgets = []
    tracking_algorithms.each do |algorithm|
      @ui.tracking_combo.addItem algorithm.description
      widget = algorithm.config_widget
      @tracking_widgets.push widget
      @ui.tracking_stack.addWidget widget
    end

    # Connect controls.
    connect @ui.model_list, SIGNAL('currentRowChanged(int)'),
      self, SLOT('select_model(int)')
    connect @ui.add_model_button, SIGNAL('clicked()'), self, SLOT('add()')
    connect @ui.delete_button, SIGNAL('clicked()'), self, SLOT('delete()')
    connect @ui.name_edit, SIGNAL('textChanged(QString)'),
      self, SLOT('model_name(QString)')
    connect @ui.add_tracking_button, SIGNAL('clicked()'),
      self, SLOT('add_tracking()')
    connect @ui.delete_tracking_button, SIGNAL('clicked()'),
      self, SLOT('delete_tracking()')
    connect @coord_widget, SIGNAL('selected()'), self, SLOT('model_coord()')
  end

  def models=( models )
    @models = models
    @ui.model_list.clear
    @models.each { |model| @ui.model_list.addItem model.name }
  end

  def image=( image )
    @recognition_widgets.each { |widget| widget.image = image }
    @tracking_widgets.each { |widget| widget.image = image }
    @coord_widget.write image
  end

  def select_model( index )
    @ui.delete_button.enabled = index >= 0
    @ui.name_edit.enabled = index >= 0
    if index >= 0
      model = @models[ index ]
      @ui.name_edit.text = model.name
      widget = @recognition_widgets[ @recognition_algorithms.
                                     index( model.
                                            recognition_algorithm.class ) ]
      widget.algorithm = model.recognition_algorithm
      @ui.recognition_stack.setCurrentWidget widget
      if model.tracking_algorithm != nil
        widget = @tracking_widgets[ @tracking_algorithms.
                                    index( model.
                                           tracking_algorithm.class ) ]
        widget.algorithm = model.tracking_algorithm
        @ui.tracking_stack.setCurrentWidget widget
        @ui.tracking_option_stack.currentIndex = 1
      else
        @ui.tracking_option_stack.currentIndex = 0
      end
      @coord_widget.select model.pose
      @ui.algorithm_tab.enabled = true
    else
      @ui.name_edit.text = "<model>"
      @ui.algorithm_tab.enabled = false
      @ui.algorithm_tab.currentIndex = 0
      @ui.recognition_stack.currentIndex = 0
      @ui.tracking_stack.currentIndex = 0
    end
  end

  def add
    @models.push Model.new( '<model>',
                            @recognition_algorithms[ @ui.recognition_combo.
                                                     currentIndex ].new )
    @ui.algorithm_tab.currentIndex = 0
    @ui.model_list.addItem @models.last.name
    @ui.model_list.currentRow = @ui.model_list.count - 1
    @ui.name_edit.selectAll
    @ui.name_edit.setFocus Qt::OtherFocusReason
  end

  def delete
    index = @ui.model_list.currentRow
    if index >= 0
      @ui.model_list.takeItem index
      @models.delete_at index
    end
  end

  def model_name( text )
    index = @ui.model_list.currentRow
    if index >= 0
      @models[ index ].name = text
      @ui.model_list.item( index ).text = text
    end
  end

  def add_tracking
    model = @models[ @ui.model_list.currentRow ]
    model.tracking_algorithm = @tracking_algorithms[ @ui.tracking_combo.
                                                     currentIndex ].new
    widget = @tracking_widgets[ @ui.tracking_combo.currentIndex ]
    widget.algorithm = model.tracking_algorithm
    @ui.tracking_stack.setCurrentWidget widget
    @ui.tracking_option_stack.currentIndex = 1
  end

  def delete_tracking
    model = @models[ @ui.model_list.currentRow ]
    model.tracking_algorithm = nil
    @ui.tracking_option_stack.currentIndex = 0
  end

  def model_coord
    model = @models[ @ui.model_list.currentRow ]
    if model.recognition_algorithm != nil
      algorithm_pose = model.pose * model.recognition_algorithm.delta.inv
      model.recognition_algorithm.delta =
        algorithm_pose.inv * @coord_widget.pose
    end
    if model.tracking_algorithm != nil
      algorithm_pose = model.pose * model.tracking_algorithm.delta.inv
      model.tracking_algorithm.delta =
        algorithm_pose.inv * @coord_widget.pose
    end
    model.pose = @coord_widget.pose
  end

end
